==> aerospike-client-csharp-3.8.0.zip: aerospike-client-csharp-3.8.0/Framework/AerospikeClient/Query/QueryAggregateCommand.cs <==
/* 
 * Copyright 2012-2019 Aerospike, Inc.
 *
 * Portions may be licensed to Aerospike, Inc. under one or more contributor
 * license agreements.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
using System;
using System.Threading;
using System.Collections.Concurrent;

namespace Aerospike.Client
{
	public sealed class QueryAggregateCommand : MultiCommand
	{
		private readonly QueryPolicy policy;
		private readonly Statement statement;
		private readonly BlockingCollection<object> inputQueue;
		private readonly CancellationToken cancelToken;

		public QueryAggregateCommand
		(
			Node node,
			QueryPolicy policy,
			Statement statement,
			BlockingCollection<object> inputQueue,
			CancellationToken cancelToken,
			ulong clusterKey,
			bool first
		) : base(node, statement.ns, clusterKey, first)
		{
			this.policy = policy;
			this.statement = statement;
			this.inputQueue = inputQueue;
			this.cancelToken = cancelToken;
		}

		protected internal override void WriteBuffer()
		{
			SetQuery(policy, statement, false);
		}

		protected internal override void ParseRow(Key key)
		{
#if NETFRAMEWORK
			if (opCount != 1)
			{
				throw new AerospikeException("Query aggregate expected exactly one bin.  Received " + opCount);
			}

			// Parse aggregateValue.
			ReadBytes(8);
			int opSize = ByteUtil.BytesToInt(dataBuffer, 0);
			byte particleType = dataBuffer[5];
			byte nameSize = dataBuffer[7];

			ReadBytes(nameSize);
			string name = ByteUtil.Utf8ToString(dataBuffer, 0, nameSize);

			int particleBytesSize = (int)(opSize - (4 + nameSize));
			ReadBytes(particleBytesSize);

			if (! name.Equals("SUCCESS"))
			{
				if (name.Equals("FAILURE"))
				{
					object value = ByteUtil.BytesToParticle(particleType, dataBuffer, 0, particleBytesSize);
					throw new AerospikeException(ResultCode.QUERY_GENERIC, value.ToString());
				}
				else
				{
					throw new AerospikeException(ResultCode.PARSE_ERROR, "Query aggregate expected bin name SUCCESS.  Received " + name);
				}
			}

			object aggregateValue = LuaInstance.BytesToLua(particleType, dataBuffer, 0, particleBytesSize);

			if (!valid)
			{
				throw new AerospikeException.QueryTerminated();
			}

			if (aggregateValue != null)
			{
				try
				{
					inputQueue.Add(aggregateValue, cancelToken);
				}
				catch (OperationCanceledException)
				{
				}
			}
#endif
		}
	}
}

==> aerospike-client-csharp-3.9.3.zip: aerospike-client-csharp-3.9.3/Framework/AerospikeClient/Query/QueryAggregateCommand.cs <==
/* 
 * Copyright 2012-2019 Aerospike, Inc.
 *
 * Portions may be licensed to Aerospike, Inc. under one or more contributor
 * license agreements.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
using System;
using System.Threading;
using System.Collections.Concurrent;

namespace Aerospike.Client
{
	public sealed class QueryAggregateCommand : MultiCommand
	{
		private readonly QueryPolicy policy;
		private readonly Statement statement;
		private readonly BlockingCollection<object> inputQueue;
		private readonly CancellationToken cancelToken;

		public QueryAggregateCommand
		(
			Node node,
			QueryPolicy policy,
			Statement statement,
			BlockingCollection<object> inputQueue,
			CancellationToken cancelToken,
			ulong clusterKey,
			bool first
		) : base(node, statement.ns, clusterKey, first)
		{
			this.policy = policy;
			this.statement = statement;
			this.inputQueue = inputQueue;
			this.cancelToken = cancelToken;
		}

		protected internal override void WriteBuffer()
		{
			SetQuery(policy, statement, false);
		}

		protected internal override void ParseRow(Key key)
		{
#if NETFRAMEWORK
			if (opCount != 1)
			{
				throw new AerospikeException("Query aggregate expected exactly one bin.  Received " + opCount);
			}

			// Parse aggregateValue.
			int opSize = ByteUtil.BytesToInt(dataBuffer, dataOffset);
			dataOffset += 5;
			byte particleType = dataBuffer[dataOffset];
			dataOffset += 2;
			byte nameSize = dataBuffer[dataOffset++];
			string name = ByteUtil.Utf8ToString(dataBuffer, dataOffset, nameSize);
			dataOffset += nameSize;

			int particleBytesSize = (int)(opSize - (4 + nameSize));

			if (! name.Equals("SUCCESS"))
			{
				if (name.Equals("FAILURE"))
				{
					object value = ByteUtil.BytesToParticle(particleType, dataBuffer, dataOffset, particleBytesSize);
					throw new AerospikeException(ResultCode.QUERY_GENERIC, value.ToString());
				}
				else
				{
					throw new AerospikeException(ResultCode.PARSE_ERROR, "Query aggregate expected bin name SUCCESS.  Received " + name);
				}
			}

			object aggregateValue = LuaInstance.BytesToLua(particleType, dataBuffer, dataOffset, particleBytesSize);
			dataOffset += particleBytesSize;

			if (!valid)
			{
				throw new AerospikeException.QueryTerminated();
			}

			if (aggregateValue != null)
			{
				try
				{
					inputQueue.Add(aggregateValue, cancelToken);
				}
				catch (OperationCanceledException)
				{
				}
			}
#endif
		}
	}
}
